@@ -1,56 +1,111 @@
 /*
-    NodeTree - Pythonic XML Data Binding Package
-    Copyright (C) 2010,2011 Arc Riley
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published
-    by the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, see http://www.gnu.org/licenses
-*/
+ *  NodeTree - Pythonic XML Data Binding Package
+ *  Copyright (C) 2010,2011,2012,2013 Arc Riley
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published
+ *  by the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public License
+ *  along with this program; if not, see http://www.gnu.org/licenses
+ *
+ */
 
 
 #include "nodetree.h"
+#define SELF nodetree_Document_Object*
 
 
 static int
-root_setter (nodetree_Document_Object* self, PyObject* value, void* closure);
+root_setter (SELF self, PyObject* value, void* closure);
 
 
 static char
-tp_doc[] = "NodeTree Document\n"
+tp_doc[] = ".. py:class:: Document([xml])\n"
 "\n"
-"    Documents are XML streams which contain exactly one root element and may\n"
-"    contain comments, processing instructions, and other types of XML nodes.\n"
+"    Documents are XML streams which contain one root :py:class:`Element`\n"
+"    node and may contain one or more :py:class:`Comment` nodes,\n"
+"    :py:class:`ProcessingInstruction` nodes, and other XML nodes types.\n"
 "\n"
-"    As with Element, Document children are accessed as items.  While there\n"
-"    may be only one root element in a document, the root element is not\n"
-"    guarenteed to be the first item (and will often not be).\n"
+"    Document children can be added with the append method and accessed or\n"
+"    replaced as items::\n"
 "\n"
-"    The .name property holds the Document's name, filename, or URI.\n"
-"    The .root property is a shortcut to the Document's root element.\n"
-"    The .version property is the XML version used (currently '1.0' only)\n"
+"        >>> doc = nodetree.Document()\n"
+"        >>> doc.append(nodetree.Comment(' Start '))\n"
+"        >>> doc.append(nodetree.Element('data'))\n"
+"        >>> doc.append(nodetree.Comment(' Fini '))\n"
+"        >>> doc[0]\n"
+"        <!-- Start -->\n"
+"        >>> doc[-1] = nodetree.Comment(' End ')\n"
+"        >>> doc\n"
+"        <?xml version=\"1.0\"?>\n"
+"        <!-- Start -->\n"
+"        <data/>\n"
+"        <!-- End -->\n"
+"\n"
+"    You may also parse an existing XML document by passing it as a string\n"
+"    argument at instantation::\n"
+"\n"
+"        >>> doc = nodetree.Document('<!-- Start --><data/><!-- End -->')\n"
+"        >>> doc\n"
+"        <?xml version=\"1.0\"?>\n"
+"        <!-- Start -->\n"
+"        <data/>\n"
+"        <!-- End -->\n"
+"\n"
+"    :param string xml: XML to be parsed into new document\n"
 "\n";
 
 
 static PyObject*
 tp_new (PyTypeObject* type, PyObject* args, PyObject* kwds) {
-    nodetree_Document_Object* self;
+    SELF self;
+    char* data = NULL;
+    int size = 0;
+
+    // Ensure no keywords were given
+    if (!_PyArg_NoKeywords("Document", kwds))
+        return NULL;
+
+    // Parse the string
+    if (!PyArg_ParseTuple(args, "|z#:Document", &data, &size))
+        return NULL;
 
     // Inherit base type
-    self = (nodetree_Document_Object*) PyType_GenericNew(type, args, kwds);
+    self = (SELF) PyType_GenericNew(type, args, kwds);
     if (!self)
       return NULL;
 
-    // Create new XML document
-    self->node = xmlNewDoc("1.0");
+    // Set self->node initially to NULL
+    self->node = NULL;
+
+    // Parse string if provided
+    if (size) {
+        xmlParserCtxt* ctxt;
+
+        // Create parser for provided XML string
+        ctxt = xmlCreatePushParserCtxt(NULL, NULL, data, size, NULL);
+
+        // Mark parser for document termination
+        xmlParseChunk(ctxt, NULL, 0, 1);
+
+        // Link new XML document to this object if it exists
+        if (ctxt->myDoc)
+            self->node = ctxt->myDoc;
+
+        // Free parser context
+        xmlFreeParserCtxt(ctxt);
+    }
+
+    // Create empty document if parser didn't create it already
+    if (!self->node)
+        self->node = xmlNewDoc((xmlChar*) "1.0");
 
     // Link new XML document to this object
     self->node->_private = self;
@@ -61,9 +116,10 @@ tp_new (PyTypeObject* type, PyObject* args, PyObject* kwds) {
 
 
 static void
-tp_dealloc (nodetree_Document_Object* self) {
-    // Free the XML document
-    xmlFreeDoc(self->node);
+tp_dealloc (SELF self) {
+    // Free the XML document if it exists
+    //if (self->node)
+    //    xmlFreeDoc(self->node);
 
     // Dealloc base type
     PyObject_Del((PyObject*) self);
@@ -71,16 +127,18 @@ tp_dealloc (nodetree_Document_Object* self) {
 
 
 static PyObject*
-tp_repr (nodetree_Document_Object* self) {
+tp_repr (SELF self) {
     xmlChar* string;
-    int string_size;
+    int string_size = 1; // 1-1 = 0, see a few lines below
     PyObject* ret;
 
     // Dump XML for entire document to a string
-    xmlDocDumpFormatMemory(self->node, &string, &string_size, 1);
+    if (self->node)
+        xmlDocDumpFormatMemory(self->node, &string, &string_size, 1);
 
     // Convert the returned XML string to a Python return string
-    ret = PyUnicode_FromStringAndSize((char*) string, string_size);
+    // We're chopping the last \n off the returned string with string_size - 1
+    ret = PyUnicode_FromStringAndSize((char*) string, string_size - 1);
 
     // Free XML string and return the Python string
     xmlFree(string);
@@ -89,13 +147,14 @@ tp_repr (nodetree_Document_Object* self) {
 
 
 static PyObject*
-tp_str (nodetree_Document_Object* self) {
+tp_str (SELF self) {
     xmlChar* string;
-    int string_size;
+    int string_size = 0;
     PyObject* ret;
 
     // Dump XML for entire document to a string
-    xmlDocDumpMemory(self->node, &string, &string_size);
+    if (self->node)
+        xmlDocDumpMemory(self->node, &string, &string_size);
 
     // Convert the returned XML string to a Python return string
     ret = PyUnicode_FromStringAndSize((char*) string, string_size);
@@ -110,26 +169,38 @@ tp_str (nodetree_Document_Object* self) {
 // Sequence Methods
 
 static Py_ssize_t
-sq_length (nodetree_Document_Object* self) {
+sq_length (SELF self) {
+    // If document node doesn't exist yet, return 0
+    if (!self->node)
+        return 0;
+
     // Use child node counter
     return (Py_ssize_t) nodetree_node_Child_Count((xmlNode*) self->node);
 }
 
 
 static PyObject*
-sq_item (nodetree_Document_Object* self, Py_ssize_t index) {
+sq_item (SELF self, Py_ssize_t index) {
+    // Raise error if document node doesn't exist yet
+    if (!self->node) {
+        PyErr_SetString(PyExc_IndexError, "Child index out of range");
+        return NULL;
+    }
+
     // Use child node getter
     return nodetree_node_Child_Get((xmlNode*) self->node, (int) index);
 }
 
 
 static int
-sq_ass_item (nodetree_Document_Object* self, Py_ssize_t index,
+sq_ass_item (SELF self, Py_ssize_t index,
              PyObject* value) {
     int i = 0;
     xmlNode* child;
     xmlNode* oldroot;
 
+    // TODO ensure document node exists before trying to use it
+
     /* Element node handling
 
         We need to handle Element children specially because only one is
@@ -153,7 +224,8 @@ sq_ass_item (nodetree_Document_Object* self, Py_ssize_t index,
             need to be.  Code complexity is better here to keep the API simple.
         */
 
-        if (oldroot = xmlDocGetRootElement(self->node)) {
+        oldroot = xmlDocGetRootElement(self->node);
+        if (oldroot) {
             // Get first child
             child = self->node->children;
 
@@ -168,9 +240,7 @@ sq_ass_item (nodetree_Document_Object* self, Py_ssize_t index,
             // Ensure the new root and old root are at the same index
             if (i != index) {
                 PyErr_SetString(PyExc_ValueError,
-                                "Can only replace root node in-place.\n"
-                                "Use Document.root property to replace the "
-                                "root node or delete it first.");
+                                "Document already contains a root Element");
                 return -1;
             }
         }
@@ -207,22 +277,28 @@ sq_ass_item (nodetree_Document_Object* self, Py_ssize_t index,
 
 
 static int
-sq_contains (nodetree_Document_Object* self, PyObject* value) {
+sq_contains (SELF self, PyObject* value) {
     xmlNode* child;
     nodetree_node* next;
 
+    // False if document node doesn't exist yet
+    if (!self->node)
+        return 0;
+
     // Documents only have one root Element, so this is easy to check
     if (nodetree_Element_Check(value)) {
         child = xmlDocGetRootElement(self->node);
 
         // Check first node
-        if (child = ((nodetree_Element_Object*) value)->node)
+        child = ((nodetree_Element_Object*) value)->node;
+        if (child)
             return 1;
 
         // Then search node list for a match
         next = ((nodetree_Element_Object*) value)->next;
         while (next) {
-            if (child = next->node)
+            child = next->node;
+            if (child)
                 return 1;
             next = (nodetree_node*) next->next;
         }
@@ -242,20 +318,41 @@ sq_contains (nodetree_Document_Object* self, PyObject* value) {
 // General Methods
 
 static char
-append_doc[] = "NodeTree Document.append method\n"
+append_doc[] = ".. py:method:: Document.append(node)\n"
 "\n"
-"    This method can be used to add a child node to the document.  While an\n"
-"    XML document may only have one root Element, other types of nodes can\n"
-"    be added, in any order, such as comments and processing instructions.\n"
+"    This method can be used to add a child node to the document.\n"
 "\n"
-"    If a root Element is added to a Document which already has one, the old\n"
-"    root Element will be detached and will also be freed from memory if no\n"
-"    reference to it is held by Python.\n"
+"    While an XML document may only have one root :py:class:`Element`\n"
+"    node, :py:class:`Comment` and :py:class:`ProcessingInstruction`\n"
+"    nodes may be added before and/or after the root element.\n"
+"\n"
+"    Attempting to append a node which would break XML specification\n"
+"    will raise an exception::\n"
+"\n"
+"        >>> doc = nodetree.Document()\n"
+"        >>> doc.append(nodetree.Comment(' Before '))\n"
+"        >>> doc.append(nodetree.Element('library')) # Root element\n"
+"        >>> doc.append(nodetree.Comment(' After '))\n"
+"        >>> doc.append('text node')\n"
+"        Traceback (most recent call last):\n"
+"            ...\n"
+"        TypeError: Text nodes can only be added to an Element node.\n"
+"        >>> doc.append(nodetree.Element('collection'))\n"
+"        Traceback (most recent call last):\n"
+"            ...\n"
+"        ValueError: Document already contains a root Element\n"
+"        >>> doc\n"
+"        <?xml version=\"1.0\"?>\n"
+"        <!-- Before -->\n"
+"        <library/>\n"
+"        <!-- After -->\n"
+"\n"
+"    :param node: node to be added to the document\n"
 "\n";
 
 
 static PyObject*
-append (nodetree_Document_Object* self, PyObject* args) {
+append_meth (SELF self, PyObject* args) {
     PyObject* item;
     xmlNode* child;
 
@@ -289,18 +386,15 @@ append (nodetree_Document_Object* self, PyObject* args) {
         */
 
         if (xmlDocGetRootElement(self->node)) {
-            PyErr_SetString(PyExc_ValueError, "Cannot add second root node.\n"
-                            "Use Document.root property to replace the root "
-                            "node or delete it first.");
+            PyErr_SetString(PyExc_ValueError,
+                            "Document already contains a root Element");
             return NULL;
         }
 
         // If there is not already a root node, this behaves exactly like using
         // the Document.root property, so we just call that code directly.
-        if (root_setter(self, item, NULL) == -1) {
-            // Exception set by root_setter already
+        if (root_setter(self, item, NULL) == -1)
             return NULL;
-        }
 
         // Returns None on success
         Py_RETURN_NONE;
@@ -344,11 +438,112 @@ append (nodetree_Document_Object* self, PyObject* args) {
 }
 
 
+static char
+pop_doc[] = ".. py:method:: Document.pop([index])\n"
+"\n"
+"    Remove child node by index.\n"
+"\n"
+"    :param int index: index of target child (defaults to last)\n"
+"    :returns: removed child node\n"
+"\n";
+
+static PyObject*
+pop_meth (SELF self, PyObject* args) {
+    int index = -1;
+    xmlNode* item;
+    PyObject* child;
+
+    // Parse item as single argument
+    if (!PyArg_ParseTuple(args, "|i", &index)) {
+        return NULL;
+    }
+
+    // Ensure there's at least one child
+    if (self->node->children == NULL) {
+        PyErr_SetString(PyExc_IndexError, "pop from empty Document");
+        return NULL;
+    }
+
+    // Get either last child node or child node by index
+    if (index == -1)
+        item = self->node->last;
+    else
+        item = nodetree_node_Child_Index((xmlNode*) self->node, index);
+
+    // Raise exception if child node wasn't found
+    if (!item) {
+        PyErr_SetString(PyExc_IndexError, "child index out of range");
+        return NULL;
+    }
+
+    // Check if this is an active stream
+    if (nodetree_Stream_Check(self) &&
+        ((nodetree_Stream_Object*) self)->ctxt) {
+        // We need to make sure the node being popped is not part of the
+        // branch currently being parsed by the stream
+        xmlNode* current = ((nodetree_Stream_Object*) self)->ctxt->node;
+
+        // Walk up tree ensuring no ancestor of current node is our item
+        while (current && current != (xmlNode*) self->node) {
+            if (current == item) {
+                PyErr_SetString(PyExc_ValueError, "incomplete branch");
+                return NULL;
+            }
+            current = current->parent;
+        }
+
+    }
+
+    // Get child object for item
+    child = nodetree_node_Create(item);
+
+    // Prune child
+    nodetree_node_Prune(item);
+
+    // Return popped child
+    return child;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // Properties
 
+static char
+root_doc[] = ".. py:attribute:: Document.root\n"
+"\n"
+"    While XML requires only one root :py:class:`Element` per document,\n"
+"    it is not guarenteed to be the first child node of a document.\n"
+"\n"
+"    This property provides easy access to the root element regardless\n"
+"    of its position in the document.\n"
+"\n"
+"    This property may be used to replace the root element in-place or\n"
+"    delete the old root before adding a new root element::\n"
+"\n"
+"        >>> doc = nodetree.Document()\n"
+"        >>> doc.append(nodetree.Comment(' Start '))\n"
+"        >>> doc.append(nodetree.Element('catalog'))\n"
+"        >>> doc.append(nodetree.Comment(' End '))\n"
+"        >>> doc.root\n"
+"        <catalog/>\n"
+"        >>> doc.root = nodetree.Element('library')\n"
+"        >>> doc\n"
+"        <?xml version=\"1.0\"?>\n"
+"        <!-- Start -->\n"
+"        <library/>\n"
+"        <!-- End -->\n"
+"        >>> del(doc.root)\n"
+"        >>> doc.root = nodetree.Element('collection')\n"
+"        >>> doc\n"
+"        <?xml version=\"1.0\"?>\n"
+"        <!-- Start -->\n"
+"        <!-- End -->\n"
+"        <collection/>\n"
+"\n";
+
+
 static PyObject*
-root_getter (nodetree_Document_Object* self, void* closure) {
+root_getter (SELF self, void* closure) {
     xmlNode* root;
 
     // Get root element from libxml2
@@ -358,32 +553,27 @@ root_getter (nodetree_Document_Object* self, void* closure) {
     if (!root)
         Py_RETURN_NONE;
 
+    // Incref existing Element object if there is one
     if (root->_private)
-        // Incref existing Element object if there is one
-        Py_INCREF((PyObject*) root->_private);
-    else
-        // Create a new Element object to wrap the node for Python
-        nodetree_Element_Create(root);
+        return Py_INCREF(root->_private), root->_private;
 
-    return (PyObject*) root->_private;
+    // Create a new Element object to wrap the node for Python
+    return (PyObject*) nodetree_Element_Create(root);
 }
 
 
 static int
-root_setter (nodetree_Document_Object* self, PyObject* value, void* closure) {
-    nodetree_Element_Object* element;
-    xmlNode* oldroot;
-    xmlNode* newroot;
+root_setter (SELF self, PyObject* value, void* closure) {
+    nodetree_Element_Object* element = (void*) value; // for code simplicity
+    xmlNode* root;
 
     // Handle del(document.root) and document.root = None calls
     if (!value || value == Py_None) {
-        oldroot = xmlDocGetRootElement(self->node);
+        root = xmlDocGetRootElement(self->node);
 
-        // Do nothing if there is no root element
-        if (oldroot) {
-            // Prune node from tree
-            nodetree_node_Prune(oldroot);
-        }
+        // Prune oldroot if currently set
+        if (root)
+            nodetree_node_Prune(root);
 
         // Always succeeds
         return 0;
@@ -391,25 +581,20 @@ root_setter (nodetree_Document_Object* self, PyObject* value, void* closure) {
 
     // Ensure value is an Element
     if (!nodetree_Element_Check(value)) {
-        PyErr_SetString(PyExc_TypeError,
-                        "root attribute must be an Element node");
+        PyErr_SetString(PyExc_TypeError, "root attribute must be an Element");
         return -1;
     }
 
-    // This is just for code simplicity
-    element = (nodetree_Element_Object*) value;
-
     // Check to see if Element's primary node is in use
     if (element->node->parent)
-        nodetree_node_Copy(element);
+        nodetree_node_Copy(element->node);
 
-    // This function returns the old root element (or NULL)
-    oldroot = xmlDocSetRootElement(self->node,
-                                   ((nodetree_Element_Object*) value)->node);
+    // This function returns the old root element (or NULL if none)
+    root = xmlDocSetRootElement(self->node, element->node);
 
     // Clean up old root if necessary
-    if (oldroot)
-        nodetree_node_Prune(oldroot);
+    if (root)
+        nodetree_node_Prune(root);
 
     // Return success
     return 0;
@@ -435,9 +620,13 @@ static PySequenceMethods tp_as_sequence = {
 
 static PyMethodDef tp_methods[] = {
     {"append",                                             // ml_name
-     (PyCFunction) append,                                 // ml_meth
+     (PyCFunction) append_meth,                            // ml_meth
      METH_VARARGS,                                         // ml_flags
      append_doc},                                          // ml_doc
+    {"pop",                                                // ml_name
+     (PyCFunction) pop_meth,                               // ml_meth
+     METH_VARARGS,                                         // ml_flags
+     pop_doc},                                             // ml_doc
     {NULL},                                                // sentinel
 };
 
@@ -446,7 +635,7 @@ static PyGetSetDef tp_getset[] = {
     {"root",
      (getter) root_getter,
      (setter) root_setter,
-     "Document's root element - there can be only one.",
+     root_doc,
      NULL},
     {NULL},                                                // sentinel
 };
